/*
 * generated by Xtext 2.35.0
 */
package org.tfranke.webservice.dsl.generator

import org.eclipse.emf.ecore.resource.Resource
import org.eclipse.xtext.generator.AbstractGenerator
import org.eclipse.xtext.generator.IFileSystemAccess2
import org.eclipse.xtext.generator.IGeneratorContext
import webservice.API
import webservice.Operation	
import webservice.ReadOperation
import java.util.Map
import java.util.List
import java.util.HashMap
import webservice.Parameter
import webservice.CreateOperation
import webservice.UpdateOperation
import webservice.DeleteOperation
import webservice.DataType
import webservice.StringData
import webservice.IntegerData
import webservice.BooleanData
import webservice.FloatData
import webservice.CollectionDataType
import webservice.ObjectDataType
import com.fasterxml.jackson.databind.ObjectMapper
import java.util.ArrayList
import java.util.regex.Pattern
import java.util.regex.Matcher

/**
 * Generates code from your model files on save.
 * 
 * See https://www.eclipse.org/Xtext/documentation/303_runtime_concepts.html#code-generation
 */
class WebserviceDSLGenerator extends AbstractGenerator {
	
	override void doGenerate(Resource res, IFileSystemAccess2 fsa, IGeneratorContext context ) { 
		for (API api : res.allContents.toIterable.filter(API)){ 
			fsa.generateFile( "openAPI/" + api.name + ".json", formatJson(generateOpenApi(api)))
			TransconnectRepositoryCreator.createRepository(api, fsa) //creates TC Repository
		}
	}
	
	/**
	 * This methods creates an openApi specification in Json form from an API Object.
	 */
	def String generateOpenApi(API api) {
		val List<DataType> complexTypes = new ArrayList()
		for (DataType datatype : api.datatypes) {
			if (datatype instanceof ObjectDataType || datatype instanceof CollectionDataType) {
				complexTypes.add(datatype)
			}
		}
		
	'''	
	{
	  "openapi": "3.0.0",
	  "info": {
	  	"title": "«api.name»",
	  	"description": "«api.description»",
	    "version": "1.0.0"
	  },
	  "servers": [
	    {
	      "url": "«api.url»",
	      "description": "«api.description»"
	    }
	  ],
	  "paths": {
	    «createEndpoints(api)»
	  }«IF !complexTypes.isEmpty»,
	"components" : {
		"schemas": {
			«FOR DataType datatype : complexTypes SEPARATOR ','»
			«IF datatype instanceof ObjectDataType»
			«constructObjectDataTypeSchema(datatype)»
			«ELSEIF datatype instanceof CollectionDataType»
			«constructArrayDataTypeSchema(datatype)»
			«ENDIF»
			«ENDFOR»
			}
		}
		«ENDIF»
	}		
	'''
	}
	
	
	/**
	 * This method creates the openApi Json representation of a schema of an object datatype.
	 */
	def String constructObjectDataTypeSchema(ObjectDataType object) {
		val properties = object.datatypes
		val List<DataType> requiredProperties = new ArrayList()
		for(property : properties) {
			if (property.required) {
				requiredProperties.add(property)
			}
		}
		
		'''
		"«object.name»": {
	        "type": "object",
	        "properties": {
	          «FOR DataType property : properties SEPARATOR ','»
	          "«property.name»": {
	           	«getTypeOfData(property)»
	          }
	          «ENDFOR»
	        }«IF !requiredProperties.isEmpty»,
	        "required": [
	        «FOR DataType property : requiredProperties SEPARATOR ','»
	        "«property.name»"
	         «ENDFOR»
	        ]
	        «ENDIF»
        }
		'''
	}
	
	/**
	 * This method creates the openApi Json representation of a schema of an array datatype.
	 */
	def constructArrayDataTypeSchema(CollectionDataType datatype) {
		'''
		"«datatype.name»": {
	        "type": "array",
	        "items": {
	          «getTypeOfData(datatype.listItem)»
	        }
	      }
		'''
	}
	
	/**
	 * This method creates the openApi Json representation of an endpoint. 
	 */
	def createEndpoints(API api) {
		val urlToOperation = mapUrlToOperations(api)
		
		'''
		«FOR String url : urlToOperation.keySet SEPARATOR ','»	
		"«url»": {
		  «FOR Operation operation : urlToOperation.get(url) SEPARATOR ','»
		  "«getHttpMethodFromOperation(operation)»": {
		    "summary": "«operation.description»",
		    "operationId": "«operation.name»",
		    «handleOperationParameters(operation, url)»
		    «IF operation.response !== null »,
		    "responses": {
		      "200": {
		      	"description": "OK",
		        "content": {
		          "*/*": {
		            "schema": {
	        	      «getTypeOfData(operation.response)»
		            }
		          }
		        }
		      }
		    }
	      	«ENDIF»
		  }
	      «ENDFOR»
		}
	    «ENDFOR»
		'''
	}
	
	/**
	 * Handles all request parameters for the given operation. Evaluates if: parameters belong to url path, 
	 * if they are query parameters, or if they belong to the "requestBody". If there are multiple "requestBody" parameters they 
	 * will be included in a "anyOf" tag.
	 */
	def String handleOperationParameters(Operation operation, String url) {
		//typically objects and arrays belong to the request body in post, put, delete
		val List<Parameter> requestBodyData = new ArrayList()
		val List<Parameter> parameterData = new ArrayList()
		
		//check if operation is http get or delete method
		if(operation instanceof ReadOperation || operation instanceof DeleteOperation) {
			parameterData.addAll(operation.parameters)	
		}
		//if post, put or delete: distinguish between request body and normal parameters
		else {
			for (Parameter parameter : operation.parameters) {
				val DataType datatype = parameter.datatype
				if (datatype instanceof ObjectDataType || datatype instanceof CollectionDataType) {
					requestBodyData.add(parameter)
				}
				else {
					parameterData.add(parameter)
				}
			}
		}
		
		'''
		«IF !requestBodyData.isEmpty»
		"requestBody": {
		          "content": {
		            "*/*": {
		              "schema": {
		              	«IF requestBodyData.size > 1»
		              	"anyOf": [
		              	«FOR Parameter parameter : requestBodyData SEPARATOR ','»
		                {«getTypeOfData(parameter.datatype)»}
		                «ENDFOR»
		                ]
		                «ELSE»
		                «getTypeOfData(requestBodyData.get(0).datatype)»
		                «ENDIF»       
		              }
		            }
		          }
		        }
		«ENDIF»
		«IF !parameterData.isEmpty»«IF !requestBodyData.isEmpty»,«ENDIF»
		"parameters": [
		«FOR Parameter parameter : parameterData SEPARATOR ','»
	        {
	        "name": "«parameter.datatype.name»",
	        "in": "«getParameterType(parameter, url)»",
	        "description": "«parameter.description»",
	        "required": «parameter.required»,
	        "schema": {
	        	«getTypeOfData(parameter.datatype)»
	        }
	      }
	      «ENDFOR»
	    ]
	    «ENDIF»
		'''
	}
	
	/**
	 * This method returns a map that contains all urls as keys and a list of Operations belonging to that url.
	 */
	def Map<String, List<Operation>> mapUrlToOperations(API api) {
	    val Map<String, List<Operation>> urlToOperationsMap = new HashMap<String, List<Operation>>()

	    for (operation : api.operations) {
	        val url = operation.urlSuffix
	        //if url already in map -> add operation to the list in map
	        if (urlToOperationsMap.containsKey(url)) {
	            urlToOperationsMap.get(url).add(operation)
	        } else {
	            //if url not in map -> add url and create new list 
	            urlToOperationsMap.put(url, newArrayList(operation))
	        }
    	}
    
    	return urlToOperationsMap
	}
	
	
	/**
	 * This method evaluates whether the given parameter is a path or query parameter.
	 */
	def String getParameterType(Parameter parameter, String url) {
		val List<String> urlParameter = new ArrayList()
        val Pattern pattern = Pattern.compile("\\{([^}]*)\\}");
        val Matcher matcher = pattern.matcher(url);
        
        while (matcher.find()) {
            urlParameter.add(matcher.group(1).toLowerCase);
        }
        
        if (urlParameter.contains(parameter.datatype.name.toLowerCase)) {
        	return "path"
        }
		
		return "query"
	}
	
	/**
	 * This method returns the associated HTTP-Method for an Operation object.
	 */
	def String getHttpMethodFromOperation(Operation operation){
		if(operation instanceof ReadOperation) {
			return "get"
		}
		if(operation instanceof CreateOperation) {
			return "post"
		}
		if(operation instanceof UpdateOperation) {
			return "put"
		}
		if(operation instanceof DeleteOperation) {
			return "delete"
		}
		return "other" //
	}
	
	/**
	 * This methods returns the openApi json representation of different types of data.
	 */
	def String getTypeOfData(DataType datatype) {
		if (datatype instanceof StringData) {
			return "\"type\": \"string\""
		}
		if (datatype instanceof IntegerData) {
			return "\"type\": \"integer\""
		}
		if (datatype instanceof BooleanData) {
			return "\"type\": \"boolean\""
		}
		if (datatype instanceof FloatData) {
			return "\"type\": \"float\""
		}
		if (datatype instanceof ObjectDataType || datatype instanceof CollectionDataType) {
			return "\"$ref\" : \"#/components/schemas/" + datatype.name + "\""
		}
	}
	
	/**
	 * This methods formats the given Json string.
	 */
	def String formatJson(String unformattedString) {
		val objectMapper = new ObjectMapper
		val jsonObject = objectMapper.readValue(unformattedString, Object)
		val prettyJson = objectMapper.writerWithDefaultPrettyPrinter().writeValueAsString(jsonObject);
	    return prettyJson;
	}
	
}
